@@ -143,11 +143,18 @@ def is_portable():
         return abs_location.startswith("/media/") or abs_location.startswith("/mnt/")
 
 
+_THONNY_VERSION = None
+
+
 def get_version():
+    global _THONNY_VERSION
+    if _THONNY_VERSION:
+        return _THONNY_VERSION
     try:
         package_dir = os.path.dirname(sys.modules["thonny"].__file__)
         with open(os.path.join(package_dir, "VERSION"), encoding="ASCII") as fp:
-            return fp.read().strip()
+            _THONNY_VERSION = fp.read().strip()
+            return _THONNY_VERSION
     except Exception:
         return "0.0.0"
 
@@ -203,11 +210,35 @@ def _check_welcome():
 
         win = FirstRunWindow(mgr)
         win.mainloop()
+
+        if win.ok and sys.platform == "darwin":
+            macos_app_path = _get_macos_app_path()
+            if macos_app_path:
+                # Shouldn't proceed to the main window in the same process, as TkAqua will crash on opening a menu
+                # or saving a file (https://github.com/thonny/thonny/issues/2860).
+                # Let's restart.
+                print("Restarting", macos_app_path)
+                os.system(f"open -n '{macos_app_path}'")
+                sys.exit(0)
+
         return win.ok
     else:
         return True
 
 
+def _get_macos_app_path() -> Optional[str]:
+    if sys.platform != "darwin":
+        return None
+    orig_argv = _get_orig_argv()
+    if not orig_argv:
+        return None
+
+    if orig_argv[0].endswith("Thonny.app/Contents/MacOS/thonny"):
+        return orig_argv[0][: -len("/Contents/MacOS/thonny")]
+
+    return None
+
+
 def launch():
     import runpy
 
@@ -414,7 +445,7 @@ def _get_orig_argv() -> Optional[List[str]]:
 
 def _configure_logging(log_file, console_level=None):
     logFormatter = logging.Formatter(
-        "%(asctime)s.%(msecs)03d %(levelname)-7s %(name)s: %(message)s", "%H:%M:%S"
+        "%(asctime)s.%(msecs)03d [%(threadName)s] %(levelname)-7s %(name)s: %(message)s", "%H:%M:%S"
     )
 
     file_handler = logging.FileHandler(log_file, encoding="UTF-8", mode="w")
