@@ -5,16 +5,20 @@ from logging import getLogger
 logger = getLogger(__name__)
 
 BASE_LANGUAGE_CODE = "en_US"
-BASE_LANGUAGE_NAME = "English"
+BASE_LANGUAGE_NAME = "English (US) [default]"
 
 # https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
 
 LANGUAGES_DICT = {
+    "be_BY": "Беларуская мова [BETA]",
+    "ca_ES": "Català",
     "cs_CZ": "Čeština [BETA]",
     "de_DE": "Deutsch",
     "et_EE": "Eesti",
+    "en_GB": "English (UK)",
     BASE_LANGUAGE_CODE: BASE_LANGUAGE_NAME,
     "es_ES": "Español",
+    "eu_ES": "Euskara [BETA]",
     "fr_FR": "Français",
     "it_IT": "Italiano",
     "lt_LT": "Lietuvių",
@@ -25,22 +29,24 @@ LANGUAGES_DICT = {
     "pl_PL": "Polski",
     "pt_PT": "Português (PT)",
     "pt_BR": "Português (BR)",
-    "ro_RO": "Român",
+    "ro_RO": "Română",
     "ru_RU": "Русский",
-    "sl_SI": "Slovène",
-    "sk_SK": "Slovenčina [BETA]",
+    "sl_SI": "Slovenščina",
+    "sk_SK": "Slovenčina",
     "fi_FI": "Suomi [BETA]",
-    "sv_SE": "Svenska [BETA]",
-    "sq_AL": "Shqip [ALPHA]",
+    "sv_SE": "Svenska [ALPHA]",
+    "sq_AL": "Shqip [BETA]",
+    "vi_VN": "Tiếng Việt",
     "tr_TR": "Türkçe [BETA]",
     "uk_UA": "Українська",
     "zh_TW": "繁體中文-TW",
     "zh_CN": "简体中文 ",
-    "ja_JP": "日本語  [ALPHA]",
+    "zh_Hans": "简体中文 (zh-Hans) [BETA]",
+    "ja_JP": "日本語",
     "hy_AM": "Հայերէն [BETA]",
-    "fa_IR": "[BETA] فارسی",
+    "fa_IR": "فارسی",
     "el_GR": "Ελληνικά",
-    "ko_KR": "한국어 [韓國語] [ALPHA]",
+    "ko_KR": "한국어 [韓國語] [BETA]",
     "ar_AR": "[BETA] عربي",
     "ta_IN": "தமிழ் [BETA]",
     "th_TH": "ภาษาไทย [BETA]",
