@@ -8,12 +8,15 @@ import subprocess
 import sys
 import threading
 import time
-from typing import Optional, Sequence, Tuple
+from logging import getLogger
+from typing import Any, Optional, Sequence, Tuple
 
 PASSWORD_METHOD = "password"
 PUBLIC_KEY_NO_PASS_METHOD = "public-key (without passphrase)"
 PUBLIC_KEY_WITH_PASS_METHOD = "public-key (with passphrase)"
 
+logger = getLogger(__name__)
+
 
 def delete_dir_try_hard(path: str, hardness: int = 5) -> None:
     # Deleting the folder on Windows is not so easy task
@@ -45,9 +48,13 @@ def running_on_linux() -> bool:
 
 
 def running_on_rpi() -> bool:
+    machine_lower = platform.uname().machine.lower()
     return running_on_linux() and (
-        platform.uname().machine.lower().startswith("arm")
+        # not great heuristics, I know
+        machine_lower.startswith("arm")
+        or machine_lower.startswith("aarch64")
         or os.environ.get("DESKTOP_SESSION") == "LXDE-pi"
+        or os.environ.get("DESKTOP_SESSION") == "LXDE-pi-wayfire"
     )
 
 
@@ -92,11 +99,31 @@ def list_volumes(skip_letters=set()) -> Sequence[str]:
             return volumes
         finally:
             ctypes.windll.kernel32.SetErrorMode(old_mode)  # @UndefinedVariable
+    if sys.platform == "linux":
+        try:
+            from dbus_next.errors import DBusError
+        except ImportError:
+            logger.info("Could not import dbus_next, falling back to mount command")
+            return list_volumes_with_mount_command()
+
+        from thonny.udisks import list_volumes_sync
+
+        try:
+            return list_volumes_sync()
+        except DBusError as error:
+            if "org.freedesktop.DBus.Error.ServiceUnknown" not in error.text:
+                raise
+            # Fallback to using the 'mount' command on Linux if the Udisks D-Bus service is unavailable.
+            return list_volumes_with_mount_command()
     else:
-        # 'posix' means we're on Linux or OSX (Mac).
+        # 'posix' means we're on *BSD or OSX (Mac).
         # Call the unix "mount" command to list the mounted volumes.
-        mount_output = subprocess.check_output("mount").splitlines()
-        return [x.split()[2].decode("utf-8") for x in mount_output]
+        return list_volumes_with_mount_command()
+
+
+def list_volumes_with_mount_command() -> Sequence[str]:
+    mount_output = subprocess.check_output("mount").splitlines()
+    return [x.split()[2].decode("utf-8") for x in mount_output]
 
 
 def get_win_volume_name(path: str) -> str:
@@ -268,6 +295,53 @@ def parse_cmd_line(s):
     return shlex.split(s, posix=True)
 
 
+def jaro_similarity(s, t):
+    """Jaro distance between two strings."""
+    # Source: https://rosettacode.org/wiki/Jaro_similarity
+    s_len = len(s)
+    t_len = len(t)
+
+    if s_len == 0 and t_len == 0:
+        return 1
+
+    match_distance = (max(s_len, t_len) // 2) - 1
+
+    s_matches = [False] * s_len
+    t_matches = [False] * t_len
+
+    matches = 0
+    transpositions = 0
+
+    for i in range(s_len):
+        start = max(0, i - match_distance)
+        end = min(i + match_distance + 1, t_len)
+
+        for j in range(start, end):
+            if t_matches[j]:
+                continue
+            if s[i] != t[j]:
+                continue
+            s_matches[i] = True
+            t_matches[j] = True
+            matches += 1
+            break
+
+    if matches == 0:
+        return 0
+
+    k = 0
+    for i in range(s_len):
+        if not s_matches[i]:
+            continue
+        while not t_matches[k]:
+            k += 1
+        if s[i] != t[k]:
+            transpositions += 1
+        k += 1
+
+    return ((matches / s_len) + (matches / t_len) + ((matches - transpositions / 2) / matches)) / 3
+
+
 def levenshtein_distance(s1, s2):
     # https://en.wikibooks.org/wiki/Algorithm_Implementation/Strings/Levenshtein_distance#Python
     if len(s1) < len(s2):
@@ -514,3 +588,36 @@ def show_command_not_available_in_flatpak_message():
         tr("This command is not available if Thonny is run via Flatpak"),
         parent=get_workbench(),
     )
+
+
+def get_menu_char():
+    if running_on_windows():
+        return "≡"  # Identical to
+    else:
+        return "☰"  # Trigram for heaven, too heavy on Windows
+
+
+def download_bytes(url: str, timeout: int = 10) -> bytes:
+    from urllib.request import Request, urlopen
+
+    req = Request(
+        url,
+        headers={
+            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:124.0) Gecko/20100101 Firefox/124.0",
+            "Accept-Encoding": "gzip, deflate",
+            "Cache-Control": "no-cache",
+        },
+    )
+    with urlopen(req, timeout=timeout) as fp:
+        if fp.info().get("Content-Encoding") == "gzip":
+            import gzip
+
+            return gzip.decompress(fp.read())
+        else:
+            return fp.read()
+
+
+def download_and_parse_json(url: str, timeout: int = 10) -> Any:
+    import json
+
+    return json.loads(download_bytes(url, timeout=timeout))
