@@ -44,7 +44,11 @@ class AboutDialog(CommonDialogEx):
                 system_desc = "Linux"
 
             if "32" not in system_desc and "64" not in system_desc:
-                system_desc += " " + self.get_os_word_size_guess()
+                system_desc += self.get_os_word_size_suffix()
+        elif sys.platform == "darwin":
+            mac_ver = platform.mac_ver()[0]
+            mac_arch = platform.mac_ver()[2]
+            system_desc = f"macOS {mac_ver} ({mac_arch})"
         else:
             release = platform.release()
             if sys.platform == "win32":
@@ -56,7 +60,7 @@ class AboutDialog(CommonDialogEx):
                 except Exception:
                     logger.exception("Could not determine Windows version")
 
-            system_desc = platform.system() + " " + release + " " + self.get_os_word_size_guess()
+            system_desc = platform.system() + " " + release + self.get_os_word_size_suffix()
 
         platform_label = ttk.Label(
             self.main_frame,
@@ -64,7 +68,7 @@ class AboutDialog(CommonDialogEx):
             text=system_desc
             + "\n"
             + "Python "
-            + get_python_version_string(maxsize=sys.maxsize)
+            + get_python_version_string()
             + "\n"
             + "Tk "
             + ui_utils.get_tk_version_str(),
@@ -115,11 +119,9 @@ class AboutDialog(CommonDialogEx):
 
         self.bind("<Return>", self.on_close, True)
 
-    def get_os_word_size_guess(self):
+    def get_os_word_size_suffix(self):
         if "32" in platform.machine() and "64" not in platform.machine():
-            return "(32-bit)"
-        elif "64" in platform.machine() and "32" not in platform.machine():
-            return "(64-bit)"
+            return " (32-bit)"
         else:
             return ""
 
