@@ -2,7 +2,7 @@
 import ast
 import builtins
 import functools
-import importlib
+import importlib.util
 import inspect
 import io
 import os.path
@@ -48,6 +48,7 @@ from thonny.common import (
     get_python_version_string,
     get_single_dir_child_data,
     path_startswith,
+    running_in_virtual_environment,
     serialize_message,
     update_system_path,
 )
@@ -61,7 +62,7 @@ _backend = None
 
 
 class MainCPythonBackend(MainBackend):
-    def __init__(self, target_cwd):
+    def __init__(self, target_cwd, options):
         report_time("Before MainBackend")
         MainBackend.__init__(self)
         report_time("After MainBackend")
@@ -70,7 +71,9 @@ class MainCPythonBackend(MainBackend):
         _backend = self
 
         self._ini = None
+        self._options = options
         self._object_info_tweakers = []
+        self._warned_shadow_casters = set()
         self._import_handlers = {}
         self._input_queue = queue.Queue()
         self._source_preprocessors = []
@@ -102,8 +105,10 @@ class MainCPythonBackend(MainBackend):
         report_time("Before loading plugins")
         execute_with_frontend_sys_path(self._load_plugins)
         report_time("After loading plugins")
+        if self._options.get("run.warn_module_shadowing", False):
+            sys.addaudithook(self.import_audit_hook)
 
-        # preceding code was run in the directory containing thonny module, now switch to provided
+        # preceding code was run in an empty directory, now switch to provided
         try:
             os.chdir(os.path.expanduser(target_cwd))
         except OSError:
@@ -115,8 +120,10 @@ class MainCPythonBackend(MainBackend):
         # ... and replace current-dir path item
         # start in shell mode (may be later switched to script mode)
         # required in shell mode and also required to overwrite thonny location dir
-        sys.path[0] = ""
+        assert "" not in sys.path  # for avoiding
+        sys.path.insert(0, "")
         sys.argv[:] = [""]  # empty "script name"
+        self._check_warn_avoided_sys_path_conflicts()
 
         if os.name == "nt":
             self._install_signal_handler()
@@ -161,6 +168,86 @@ class MainCPythonBackend(MainBackend):
     def get_main_module(self):
         return __main__
 
+    def import_audit_hook(self, event: str, args):
+        if event == "import":
+            logger.debug("detected Import event with args %r", args)
+            module_name = args[0]
+            self._check_warn_sys_path_conflict(module_name.split(".")[0])
+
+    def _check_warn_avoided_sys_path_conflicts(self):
+        if not self._options.get("run.warn_module_shadowing", False):
+            return
+
+        for name in sys.modules.keys():
+            if name != "__main__":
+                self._check_warn_sys_path_conflict(name.split(".")[0])
+
+    def _check_warn_sys_path_conflict(self, root_module_name: str):
+        user_dir = sys.path[0]
+        if user_dir == "":
+            user_dir = os.getcwd()
+
+        # rough test to see if it's worth invoking the finder
+        for ext in ["", ".py", ".pyw"]:
+            pot_path = os.path.join(user_dir, root_module_name + ext)
+            if os.path.exists(pot_path):
+                logger.debug("Found import candidate: %r", pot_path)
+                break
+        else:
+            return
+
+        # It looks like a module is importable from the script dir or current dir.
+        # Is it shadowing a library module?
+
+        current_spec = self._find_spec_ignore_loaded(root_module_name)
+
+        first_entry = sys.path[0]
+        del sys.path[0]
+        try:
+            shadowed_spec = self._find_spec_ignore_loaded(root_module_name)
+        finally:
+            sys.path.insert(0, first_entry)
+
+        if shadowed_spec is None:
+            logger.debug("No shadowed spec")
+            return
+
+        if shadowed_spec.origin == current_spec.origin:
+            logger.debug("Equal current and shadowed spec")
+            return
+
+        logger.debug("%r shadows %r", current_spec.origin, shadowed_spec.origin)
+
+        if current_spec.origin in self._warned_shadow_casters:
+            return
+
+        if root_module_name in sys.modules:
+            # i.e. the method is called for post-import warning
+            verb = "can shadow"
+        else:
+            verb = "is shadowing"
+
+        self._send_output(
+            # NB! Using backticks, because Shell would present file path in quotes as frame link
+            f"WARNING: Your `{current_spec.origin}` {verb} the library module '{root_module_name}'. Consider renaming or moving it!\n\n",
+            "stderr",
+        )
+
+        self._warned_shadow_casters.add(current_spec.origin)
+
+    def _find_spec_ignore_loaded(self, module_name):
+        if module_name not in sys.modules:
+            return importlib.util.find_spec(module_name)
+
+        old_modules = sys.modules
+        try:
+            modules_copy = old_modules.copy()
+            del modules_copy[module_name]
+            sys.modules = modules_copy
+            return importlib.util.find_spec(module_name)
+        finally:
+            sys.modules = old_modules
+
     def _read_incoming_msg_line(self) -> str:
         return self._original_stdin.readline()
 
@@ -256,8 +343,11 @@ class MainCPythonBackend(MainBackend):
 
         filename = cmd.args[0]
         if os.path.isfile(filename):
-            sys.path.insert(0, os.path.abspath(os.path.dirname(filename)))
-            __main__.__dict__["__file__"] = filename
+            abs_filename = os.path.abspath(filename)
+            sys.path.insert(0, os.path.dirname(abs_filename))
+            __main__.__dict__["__file__"] = abs_filename
+
+        self._check_warn_avoided_sys_path_conflicts()
 
     def _custom_import(self, *args, **kw):
         module = self._original_import(*args, **kw)
@@ -329,16 +419,12 @@ class MainCPythonBackend(MainBackend):
             main_dir=self._main_dir,
             sys_path=sys.path,
             usersitepackages=site.getusersitepackages() if site.ENABLE_USER_SITE else None,
+            externally_managed=self._is_externally_managed(),
             prefix=sys.prefix,
             welcome_text=f"Python {get_python_version_string()} ({sys.executable})",
             executable=sys.executable,
             exe_dirs=get_exe_dirs(),
-            in_venv=(
-                hasattr(sys, "base_prefix")
-                and sys.base_prefix != sys.prefix
-                or hasattr(sys, "real_prefix")
-                and getattr(sys, "real_prefix") != sys.prefix
-            ),
+            in_venv=running_in_virtual_environment(),
             python_version=get_python_version_string(),
             cwd=os.getcwd(),
         )
@@ -582,7 +668,6 @@ class MainCPythonBackend(MainBackend):
     def _perform_pip_operation_and_list(
         self, cmd_line: List[str]
     ) -> Tuple[int, Dict[str, DistInfo]]:
-
         extra_switches = ["--disable-pip-version-check"]
         proxy = os.environ.get("https_proxy", os.environ.get("http_proxy", None))
         if proxy:
@@ -633,7 +718,6 @@ class MainCPythonBackend(MainBackend):
         return get_single_dir_child_data(path, include_hidden)
 
     def _get_path_info(self, path: str) -> Optional[Dict]:
-
         try:
             if not os.path.exists(path):
                 return None
@@ -1000,6 +1084,22 @@ class MainCPythonBackend(MainBackend):
         if "tty_mode" in cmd:
             self._tty_mode = cmd["tty_mode"]
 
+    def _is_externally_managed(self):
+        if running_in_virtual_environment():
+            return False
+
+        import sysconfig
+
+        get_default_scheme = getattr(sysconfig, "get_default_scheme", None)
+        if get_default_scheme is None:
+            # before Python 3.10
+            get_default_scheme = getattr(sysconfig, "_get_default_scheme")
+
+        marker_path = os.path.join(
+            sysconfig.get_path("stdlib", get_default_scheme()), "EXTERNALLY-MANAGED"
+        )
+        return os.path.isfile(marker_path)
+
 
 class FakeStream:
     def __init__(self, backend: MainCPythonBackend, target_stream):
@@ -1034,6 +1134,8 @@ class FakeOutputStream(FakeStream):
         finally:
             self._backend._exit_io_function()
 
+        return len(data)
+
     def writelines(self, lines):
         try:
             self._backend._enter_io_function()
@@ -1049,6 +1151,7 @@ class FakeInputStream(FakeStream):
         self._eof = False
 
     def _generic_read(self, method, original_limit):
+        have_sent_input_request = False
         if original_limit is None:
             effective_limit = -1
         elif method == "readlines" and original_limit > -1:
@@ -1090,9 +1193,12 @@ class FakeInputStream(FakeStream):
                     break
 
                 else:
-                    self._backend.send_message(
-                        BackendEvent("InputRequest", method=method, limit=original_limit)
-                    )
+                    if not have_sent_input_request:
+                        self._backend.send_message(
+                            BackendEvent("InputRequest", method=method, limit=original_limit)
+                        )
+                        have_sent_input_request = True
+
                     msg = self._backend._fetch_next_incoming_message()
                     if isinstance(msg, InputSubmission):
                         self._buffer += msg.data
