@@ -3,7 +3,13 @@ import sys
 from logging import getLogger
 from typing import List, Optional
 
+# make sure thonny folder is in sys.path (relevant in dev)
+thonny_container = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
+if thonny_container not in sys.path:
+    sys.path.insert(0, thonny_container)
+
 import thonny
+from thonny.common import PROCESS_ACK
 from thonny.plugins.micropython.os_mp_backend import SshUnixMicroPythonBackend
 
 logger = getLogger("thonny.plugins.ev3.ev3_back")
@@ -19,6 +25,7 @@ class EV3MicroPythonBackend(SshUnixMicroPythonBackend):
 if __name__ == "__main__":
     THONNY_USER_DIR = os.environ["THONNY_USER_DIR"]
     thonny.configure_backend_logging()
+    print(PROCESS_ACK)
 
     import ast
 
