@@ -6,6 +6,7 @@ from tkinter import ttk
 
 from thonny import get_workbench
 from thonny.languages import tr
+from thonny.misc_utils import running_on_mac_os
 from thonny.ui_utils import CommonDialog, select_sequence, show_dialog
 
 # TODO - consider moving the cmd_find method to main class in order to pass the editornotebook reference
@@ -22,14 +23,13 @@ logger = getLogger(__name__)
 
 
 class FindDialog(CommonDialog):
-
     last_searched_word = None
 
     def __init__(self, master):
         padx = 15
         pady = 15
 
-        super().__init__(master, takefocus=1, background="pink")
+        super().__init__(master, skip_tk_dialog_attributes=running_on_mac_os(), takefocus=1)
         main_frame = ttk.Frame(self)
         main_frame.grid(row=1, column=1, sticky="nsew")
         self.columnconfigure(1, weight=1)
@@ -67,20 +67,23 @@ class FindDialog(CommonDialog):
         self.find_label.grid(column=0, row=0, sticky="w", padx=(padx, 0), pady=(pady, 0))
 
         # Find text field
-        initial_text = ""
+        seed_text = ""
         try:
             widget = get_workbench().focus_get()
             if isinstance(widget, tk.Text):
+                # NB! selection_get may give selection from another app,
+                # that's why I'm checkin that Text has sel tags
                 selected_lines = widget.selection_get().splitlines()
-                if selected_lines:
-                    initial_text = selected_lines[0]
+                if selected_lines and len(widget.tag_ranges("sel")) > 0:
+                    seed_text = selected_lines[0]
+                    logger.debug("Got seed text %r from widget %s", seed_text, widget)
         except Exception:
-            logger.exception("Could not get initial text")
+            logger.exception("Could not get seed text")
 
-        self.find_entry_var = tk.StringVar(value=initial_text)
+        self.find_entry_var = tk.StringVar(value=seed_text)
         self.find_entry = ttk.Entry(main_frame, textvariable=self.find_entry_var)
         self.find_entry.grid(column=1, row=0, columnspan=2, padx=(0, 10), pady=(pady, 0))
-        if FindDialog.last_searched_word is not None:
+        if not seed_text and FindDialog.last_searched_word is not None:
             self.find_entry.insert(0, FindDialog.last_searched_word)
 
         # Replace text label
@@ -208,7 +211,6 @@ class FindDialog(CommonDialog):
 
     # performs the replace operation - replaces the currently active found word with what is entered in the replace field
     def _perform_replace(self):
-
         # nothing is currently in found status
         if self.active_found_tag == None:
             return
@@ -247,7 +249,6 @@ class FindDialog(CommonDialog):
 
     # replaces all occurrences of the search string with the replace string
     def _perform_replace_all(self):
-
         tofind = self.find_entry.get()
         if len(tofind) == 0:
             self.infotext_label_var.set(tr("Enter string to be replaced."))
@@ -423,7 +424,7 @@ def load_plugin() -> None:
             editor = get_workbench().get_editor_notebook().get_current_editor()
             if editor:
                 dlg = FindDialog(editor._code_view)
-                show_dialog(dlg)
+                show_dialog(dlg, modal=False)
 
     def find_f3(event):
         if _active_find_dialog is None:
