@@ -4,14 +4,17 @@ import tkinter.font
 from tkinter import ttk
 
 import thonny
-from thonny import get_workbench, rst_utils, tktextext, ui_utils
+from thonny import get_workbench, tktextext, ui_utils
 from thonny.config import try_load_configuration
+from thonny.languages import tr
 from thonny.tktextext import TextFrame
 from thonny.ui_utils import scrollbar_style
 
 
 class HelpView(TextFrame):
     def __init__(self, master):
+        from thonny import rst_utils
+
         TextFrame.__init__(
             self,
             master,
@@ -32,6 +35,15 @@ class HelpView(TextFrame):
             get_workbench().get_option("general.language"),
             "HELP_CONTENT",
         )
+        get_workbench().bind("WorkbenchReady", self.on_workbench_ready, True)
+
+    def on_workbench_ready(self, event=None):
+        self.load_index()
+
+    def on_show(self):
+        self.load_index()
+
+    def load_index(self):
         self.load_rst_file("index.rst")
 
     def load_topic(self, topic, fragment=None):
@@ -55,7 +67,7 @@ class HelpView(TextFrame):
             rst_source = fp.read()
 
         if not filename.endswith("index.rst"):
-            rst_source = "`" + _("Home") + " <index.rst>`_\n\n" + rst_source
+            rst_source = "`" + tr("Home") + " <index.rst>`_\n\n" + rst_source
 
         self.text.append_rst(rst_source)
 
@@ -65,5 +77,5 @@ def open_help():
 
 
 def load_plugin() -> None:
-    get_workbench().add_view(HelpView, _("Help"), "ne")
-    get_workbench().add_command("help_contents", "help", _("Help contents"), open_help, group=30)
+    get_workbench().add_view(HelpView, tr("Help"), "ne")
+    get_workbench().add_command("help_contents", "help", tr("Help contents"), open_help, group=30)
