@@ -10,10 +10,15 @@ from abc import ABC
 from logging import getLogger
 from typing import Callable, Optional, Union
 
+# make sure thonny folder is in sys.path (relevant in dev)
+thonny_container = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
+if thonny_container not in sys.path:
+    sys.path.insert(0, thonny_container)
+
 import thonny
 from thonny import report_time
 from thonny.backend import SshMixin
-from thonny.common import BackendEvent, serialize_message
+from thonny.common import PROCESS_ACK, BackendEvent, serialize_message
 from thonny.plugins.micropython.bare_metal_backend import LF, NORMAL_PROMPT
 from thonny.plugins.micropython.connection import MicroPythonConnection
 from thonny.plugins.micropython.mp_back import (
@@ -148,7 +153,7 @@ class UnixMicroPythonBackend(MicroPythonBackend, ABC):
             output.append(data)
 
         report_time("befini")
-        self._forward_output_until_active_prompt(collect_output, "stdout")
+        self._process_output_until_active_prompt(collect_output, "stdout")
         self._original_welcome_text = "".join(output).replace("\r\n", "\n")
         self._welcome_text = self._tweak_welcome_text(self._original_welcome_text)
         report_time("afini")
@@ -175,9 +180,9 @@ class UnixMicroPythonBackend(MicroPythonBackend, ABC):
         self._connection.read_until(end_marker.encode("ascii"))
         self._write(EOT)
         self._connection.read_until(b"\n")
-        self._forward_output_until_active_prompt(output_consumer)
+        self._process_output_until_active_prompt(output_consumer)
 
-    def _forward_output_until_active_prompt(
+    def _process_output_until_active_prompt(
         self, output_consumer: Callable[[str, str], None], stream_name="stdout"
     ):
         INCREMENTAL_OUTPUT_BLOCK_CLOSERS = re.compile(
@@ -248,7 +253,7 @@ class UnixMicroPythonBackend(MicroPythonBackend, ABC):
 
         self._connection = self._create_connection(args)
         report_time("afconn")
-        self._forward_output_until_active_prompt(self._send_output, "stdout")
+        self._process_output_until_active_prompt(self._send_output, "stdout")
         report_time("afforv")
         self.send_message(
             BackendEvent(event_type="HideTrailingOutput", text=self._original_welcome_text)
@@ -383,6 +388,7 @@ class SshUnixMicroPythonBackend(UnixMicroPythonBackend, SshMixin):
 if __name__ == "__main__":
     THONNY_USER_DIR = os.environ["THONNY_USER_DIR"]
     thonny.configure_backend_logging()
+    print(PROCESS_ACK)
 
     import ast
 
