@@ -1,10 +1,10 @@
 import shlex
 
-from thonny.plugins.micropython.connection import MicroPythonConnection
+from .connection import MicroPythonConnection
 
 
 class SshProcessConnection(MicroPythonConnection):
-    def __init__(self, client, cwd, executable, args):
+    def __init__(self, client, cwd, cmd):
         super().__init__()
         import threading
 
@@ -13,7 +13,7 @@ class SshProcessConnection(MicroPythonConnection):
         cmd_line_str = (
             "echo $$ ;"
             + ((" cd %s  2> /dev/null ;" % shlex.quote(cwd) if cwd else ""))
-            + (" exec " + " ".join(map(shlex.quote, [executable] + args)))
+            + (" exec " + " ".join(map(shlex.quote, cmd)))
         )
         self._stdin, self._stdout, _ = self._client.exec_command(
             cmd_line_str, bufsize=0, timeout=None, get_pty=True
@@ -25,7 +25,7 @@ class SshProcessConnection(MicroPythonConnection):
         self._reading_thread = threading.Thread(target=self._listen_output, daemon=True)
         self._reading_thread.start()
 
-    def write(self, data, block_size=255, delay=0.01):
+    def write(self, data: bytes) -> int:
         if isinstance(data, str):
             data = data.encode(self.encoding)
         self._stdin.write(data)
