@@ -3,6 +3,7 @@ import threading
 from logging import DEBUG, getLogger
 from queue import Queue
 
+from ...common import execute_with_frontend_sys_path
 from .connection import MicroPythonConnection
 
 logger = getLogger(__name__)
@@ -17,18 +18,10 @@ class WebReplConnection(MicroPythonConnection):
     """
 
     def __init__(self, url, password, num_bytes_received=0):
-
         self.num_bytes_received = num_bytes_received
         super().__init__()
 
-        try:
-            import websockets  # @UnusedImport
-        except:
-            print(
-                "Can't import `websockets`. You can install it via 'Tools => Manage plug-ins'.",
-                file=sys.stderr,
-            )
-            sys.exit(1)
+        execute_with_frontend_sys_path(self._try_load_websockets)
         self._url = url
         self._password = password
         self._write_responses = Queue()
@@ -45,6 +38,17 @@ class WebReplConnection(MicroPythonConnection):
         if res != "OK":
             raise res
 
+    def _try_load_websockets(self):
+        try:
+            import websockets
+        except ImportError:
+            logger.error("Could not import websockets")
+            print(
+                "Can't import `websockets`. You can install it via 'Tools => Manage plug-ins'.",
+                file=sys.stderr,
+            )
+            sys.exit(1)
+
     def _wrap_ws_main(self):
         import asyncio
 
