@@ -1,6 +1,8 @@
 import os.path
-from thonny.tktextext import TextFrame, EnhancedText
-from thonny import get_workbench, ui_utils, THONNY_USER_DIR
+
+from thonny import THONNY_USER_DIR, get_workbench, ui_utils
+from thonny.languages import tr
+from thonny.tktextext import EnhancedText, TextFrame
 from thonny.ui_utils import TextMenu
 
 
@@ -30,7 +32,7 @@ class NotesView(TextFrame):
         if not os.path.isfile(self.filename):
             self.text.insert(
                 "1.0",
-                _(
+                tr(
                     "This box is meant for your working notes -- assignment instructions, "
                     + "code snippets, whatever.\n\n"
                     + "Everything will be saved automatically "
@@ -58,4 +60,4 @@ class NotesView(TextFrame):
 
 
 def load_plugin():
-    get_workbench().add_view(NotesView, _("Notes"), "ne", default_position_key="zz")
+    get_workbench().add_view(NotesView, tr("Notes"), "ne", default_position_key="zz")
