@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+import itertools
+import json
 import os
 import re
 import subprocess
@@ -12,12 +14,25 @@ from logging import exception, getLogger
 from os import makedirs
 from tkinter import messagebox, ttk
 from tkinter.messagebox import showerror
-from typing import Dict, List, Optional, Tuple, Union, cast
+from typing import Any, Dict, List, Optional, Tuple, Union, cast
 
 import thonny
 from thonny import get_runner, get_workbench, running, tktextext, ui_utils
-from thonny.common import DistInfo, InlineCommand, normpath_with_actual_case, path_startswith
+from thonny.common import (
+    DistInfo,
+    InlineCommand,
+    normpath_with_actual_case,
+    path_startswith,
+    running_in_virtual_environment,
+)
 from thonny.languages import tr
+from thonny.misc_utils import (
+    construct_cmd_line,
+    download_and_parse_json,
+    download_bytes,
+    jaro_similarity,
+    levenshtein_distance,
+)
 from thonny.running import InlineCommandDialog, get_front_interpreter_for_subprocess
 from thonny.ui_utils import (
     AutoScrollbar,
@@ -79,7 +94,6 @@ class PipDialog(CommonDialog, ABC):
         return tr("Uninstall")
 
     def _create_widgets(self, parent):
-
         header_frame = ttk.Frame(parent)
         header_frame.grid(
             row=1,
@@ -103,7 +117,10 @@ class PipDialog(CommonDialog, ABC):
         self.search_box.bind("<B1-Motion>", lambda _: self.search_box.focus_set())
 
         self.search_button = ttk.Button(
-            header_frame, text=self.get_search_button_text(), command=self._on_search, width=25
+            header_frame,
+            text=self.get_search_button_text(),
+            command=self._on_search,
+            width=len(self.get_search_button_text()) + 2,
         )
         self.search_button.grid(row=1, column=1, sticky="nse", padx=(self.get_small_padding(), 0))
 
@@ -459,19 +476,27 @@ class PipDialog(CommonDialog, ABC):
     ) -> Optional[str]:
         return None
 
+    def _download_package_info(self, name: str, version_str: Optional[str]) -> Dict:
+        import json
+        from urllib.request import urlopen
+
+        """Running in a background thread"""
+        # Fetch info from PyPI
+        if version_str is None:
+            url = "https://pypi.org/pypi/{}/json".format(urllib.parse.quote(name))
+        else:
+            url = "https://pypi.org/pypi/{}/{}/json".format(
+                urllib.parse.quote(name), urllib.parse.quote(version_str)
+            )
+
+        logger.info("Downloading package info from %s", url)
+        with urlopen(url) as fp:
+            return json.load(fp)
+
     def _start_show_package_info(self, name):
         self.current_package_data = None
         # Fetch info from PyPI
         self._set_state("fetching")
-        # Following fetches info about latest version.
-        # This is OK even when we're looking an installed older version
-        # because new version may have more relevant and complete info.
-        _start_fetching_package_info(
-            name,
-            url=self._get_package_metadata_url(name, None),
-            fallback_url=self._get_package_metadata_fallback_url(name, None),
-            completion_handler=self._show_package_info,
-        )
 
         self._clear_info_text()
         self.title_label["text"] = ""
@@ -510,6 +535,33 @@ class PipDialog(CommonDialog, ABC):
                 self.install_button["text"] = self.get_install_button_text()
                 self.uninstall_button.grid_remove()
 
+        # start download and polling
+        from concurrent.futures.thread import ThreadPoolExecutor
+
+        executor = ThreadPoolExecutor(max_workers=1)
+        download_future = executor.submit(self._download_package_info, name, None)
+
+        def poll_fetch_complete():
+            if download_future.done():
+                try:
+                    data = download_future.result()
+                    if "info" in data and "name" not in data["info"]:
+                        # this is the case of micropython.org/pi
+                        data["info"]["name"] = name
+                    self._show_package_info(name, data)
+                except urllib.error.HTTPError as e:
+                    self._show_package_info(
+                        name, {"info": {"name": name}, "error": str(e), "releases": {}}, e.code
+                    )
+                except Exception as e:
+                    self._show_package_info(
+                        name, {"info": {"name": name}, "error": str(e), "releases": {}}, e
+                    )
+            else:
+                get_workbench().after(200, poll_fetch_complete)
+
+        poll_fetch_complete()
+
     def _append_location_to_info_path(self, path):
         self.info_text.direct_insert("end", path)
 
@@ -526,6 +578,8 @@ class PipDialog(CommonDialog, ABC):
             self._append_info_text(s, tags)
 
         def write_att(caption, value, value_tag=None):
+            if value is None:
+                return
             write(caption + ": ", "caption")
             write(value, value_tag)
             write("\n")
@@ -550,17 +604,19 @@ class PipDialog(CommonDialog, ABC):
             return
 
         info = data["info"]
-        # NB! Json from micropython.org/pi doesn't have all the same fields as PyPI!
+        # NB! Json from micropython.org index doesn't have all the same fields as PyPI!
         self.title_label["text"] = info["name"]  # search name could have been a bit different
         latest_stable_version = _get_latest_stable_version(data["releases"].keys())
         if latest_stable_version is not None:
             write_att(tr("Latest stable version"), latest_stable_version)
         else:
-            write_att(tr("Latest version"), data["info"]["version"])
+            write_att(tr("Latest version"), info["version"])
         if "summary" in info:
             write_att(tr("Summary"), info["summary"])
         if "author" in info:
             write_att(tr("Author"), info["author"])
+        if "license" in info:
+            write_att(tr("License"), info["license"])
         if "home_page" in info:
             write_att(tr("Homepage"), info["home_page"], "url")
         if info.get("bugtrack_url", None):
@@ -625,13 +681,7 @@ class PipDialog(CommonDialog, ABC):
             return self._normalize_target_path(dist.location) != self._get_target_directory()
 
     def _normalize_name(self, name):
-        # looks like (in some cases?) pip list gives the name as it was used during install
-        # ie. the list may contain lowercase entry, when actual metadata has uppercase name
-        # Example: when you "pip install cx-freeze", then "pip list"
-        # really returns "cx-freeze" although correct name is "cx_Freeze"
-
-        # https://www.python.org/dev/peps/pep-0503/#id4
-        return re.sub(r"[-_.]+", "-", name).lower().strip()
+        return canonicalize_name(name)
 
     def _normalize_target_path(self, path: str) -> str:
         return path
@@ -644,11 +694,29 @@ class PipDialog(CommonDialog, ABC):
         self.title_label.grid()
         self.title_label["text"] = tr("Search results")
         self.info_text.direct_insert("1.0", tr("Searching") + " ...")
-        _start_fetching_search_results(query, self._show_search_results)
         if discard_selection:
             self._select_list_item(0)
 
-    def _show_search_results(self, query, results: Union[List[Dict], str]) -> None:
+        from concurrent.futures.thread import ThreadPoolExecutor
+
+        executor = ThreadPoolExecutor(max_workers=1)
+        results_future = executor.submit(self._fetch_search_results, query)
+
+        def poll_fetch_complete():
+            if results_future.done():
+                try:
+                    results = results_future.result()
+                except OSError as e:
+                    self._show_search_results(query, str(e))
+                else:
+                    self._show_search_results(query, results)
+
+            else:
+                get_workbench().after(200, poll_fetch_complete)
+
+        poll_fetch_complete()
+
+    def _show_search_results(self, query, results: Union[Dict[str, List[Dict]], str]) -> None:
         self._set_state("idle")
         self._clear_info_text()
 
@@ -662,21 +730,25 @@ class PipDialog(CommonDialog, ABC):
             self._append_info_text("Try opening the package directly:\n")
             self._append_info_text(query, ("url",))
             return
-        else:
-            results = self._tweak_search_results(results, query)
 
-        for item in results:
-            # self._append_info_text("•")
-            tags = ("url",)
-            if item["name"].lower() == query.lower():
-                tags = tags + ("bold",)
+        for source, source_results in results.items():
+            if len(results) > 1:
+                self._append_info_text(source + "\n", tags=("caption",))
 
-            self._append_info_text(item["name"], tags)
-            self._append_info_text("\n")
-            self.info_text.direct_insert(
-                "end", item.get("description", "<No description>").strip() + "\n"
-            )
-            self._append_info_text("\n")
+            for item in source_results:
+                # self._append_info_text("•")
+                tags = ("url",)
+                if item["name"].lower() == query.lower():
+                    tags = tags + ("bold",)
+
+                self._append_info_text(item["name"], tags)
+                if item.get("source"):
+                    self._append_info_text(" @ " + item["source"])
+                self._append_info_text("\n")
+                self.info_text.direct_insert(
+                    "end", (item.get("description") or "<No description>").strip() + "\n"
+                )
+                self._append_info_text("\n")
 
     def _select_list_item(self, name_or_index):
         if isinstance(name_or_index, int):
@@ -719,7 +791,7 @@ class PipDialog(CommonDialog, ABC):
         data = self.current_package_data
         name = self.current_package_data["info"]["name"]
 
-        install_args = []
+        install_args = ["--progress-bar", "off"]
         if self._use_user_install():
             install_args.append("--user")
 
@@ -833,7 +905,7 @@ class PipDialog(CommonDialog, ABC):
         self._start_update_list(name)
 
     def _get_install_file_command(self, filename, is_requirements_file):
-        args = ["install"]
+        args = []
         if self._use_user_install():
             args.append("--user")
         if is_requirements_file:
@@ -869,7 +941,6 @@ class PipDialog(CommonDialog, ABC):
     def _get_active_dist(self, name):
         normname = self._normalize_name(name)
         for key in self._active_distributions:
-
             if self._normalize_name(key) == normname:
                 return self._active_distributions[key]
 
@@ -907,8 +978,95 @@ class PipDialog(CommonDialog, ABC):
     def _is_read_only(self):
         raise NotImplementedError()
 
-    def _tweak_search_results(self, results, query):
-        return results
+    def _fetch_search_results(self, query: str) -> Dict[str, List[Dict[str, str]]]:
+        """Will be executed in background thread"""
+        return {
+            "*": self._perform_pypi_search(
+                query,
+                source=None,
+                data_url=get_workbench().get_data_url("pypi_summaries_cpython.json"),
+                common_tokens=[],
+            )
+        }
+
+    def _perform_pypi_search(
+        self, query: str, source: Optional[str], data_url: str, common_tokens: List[str]
+    ) -> List[Dict[str, str]]:
+        logger.info("Performing PyPI search for %r", query)
+
+        data = download_bytes(data_url)
+        packages: List[Dict] = json.loads(data)
+
+        canonical_query = canonicalize_name(query)
+        query_parts = canonical_query.split("-")
+        for package in packages:
+            package["score"] = compute_dist_name_similarity(
+                package["name"], query_parts, common_tokens
+            )
+
+        packages.sort(key=lambda p: p["score"], reverse=True)
+
+        if not packages or packages[0]["score"] < 1.0:
+            # test for exact match
+            try:
+                dist_data = download_dist_data_from_pypi(canonical_query, None)
+                dist_info = dist_data["info"]
+                print(dist_info)
+                packages.insert(
+                    0,
+                    {"name": dist_info["name"], "summary": dist_info.get("summary"), "score": 1.0},
+                )
+            except Exception:
+                logger.info("No luck with exact match %r", canonical_query)
+
+        return [
+            dict(
+                name=p["name"],
+                version="0",
+                description=p.get("summary"),
+                source=source,
+                distance=levenshtein_distance(query, p["name"]),
+            )
+            for p in packages[:20]
+            if p["score"] > 0.6
+        ]
+
+
+def canonicalize_name(name: str) -> str:
+    return re.sub(r"[-_.]+", "-", name).lower().strip()
+
+
+def _get_sublists_of_length(l: List[Any], n: int) -> List[List[Any]]:
+    return [l[i : i + n] for i in range(len(l) - n + 1)]
+
+
+def compute_dist_name_similarity(
+    name: str, query_parts: List[str], common_tokens: List[str]
+) -> float:
+    name_parts = (name).split("-")
+
+    for common_token in common_tokens:
+        if common_token in name_parts and common_token not in query_parts:
+            # don't penalize omitting this part
+            name_parts.remove(common_token)
+
+    common_count = min(len(query_parts), len(name_parts))
+    name_perms = list(itertools.permutations(name_parts, common_count))
+    query_perms = list(itertools.permutations(query_parts, common_count))
+
+    if len(name_perms) * len(query_perms) > 36:
+        # 36 corresponds to 3-part name and 3-part query.
+        # More than that would be too much effort. Assume correct order and match sub-lists instead.
+        name_perms = _get_sublists_of_length(name_parts, common_count)
+        query_perms = _get_sublists_of_length(query_parts, common_count)
+
+    best_score = 0
+    for name_perm, query_perm in itertools.product(name_perms, query_perms):
+        score = jaro_similarity("-".join(name_perm), "-".join(query_perm))
+        best_score = max(score, best_score)
+
+    parts_length_penalty = 1.0 - abs(len(query_parts) - len(name_parts)) * 0.05
+    return best_score * parts_length_penalty
 
 
 class BackendPipDialog(PipDialog):
@@ -981,6 +1139,7 @@ class BackendPipDialog(PipDialog):
             title=command,
             instructions=title,
             autostart=True,
+            output_prelude=f"{command} {construct_cmd_line(args)}\n",
         )
         ui_utils.show_dialog(dlg)
 
@@ -1044,13 +1203,7 @@ class PluginsPipDialog(PipDialog):
         return not self._targets_virtual_environment()
 
     def _targets_virtual_environment(self):
-        # https://stackoverflow.com/a/42580137/261181
-        return (
-            hasattr(sys, "base_prefix")
-            and sys.base_prefix != sys.prefix
-            or hasattr(sys, "real_prefix")
-            and getattr(sys, "real_prefix") != sys.prefix
-        )
+        return running_in_virtual_environment()
 
     def _confirm_install(self, package_data):
         name = package_data["info"]["name"]
@@ -1418,29 +1571,7 @@ def _start_fetching_package_info(name, url: str, fallback_url: str, completion_h
     poll_fetch_complete()
 
 
-def _start_fetching_search_results(query, completion_handler):
-    import urllib.parse
-
-    url = "https://pypi.org/search/?q={}".format(urllib.parse.quote(query))
-
-    url_future = _fetch_url_future(url)
-
-    def poll_fetch_complete():
-
-        if url_future.done():
-            try:
-                _, bin_data = url_future.result()
-                raw_data = bin_data.decode("UTF-8")
-                completion_handler(query, _extract_search_results(raw_data))
-            except Exception as e:
-                completion_handler(query, str(e))
-        else:
-            get_workbench().after(200, poll_fetch_complete)
-
-    poll_fetch_complete()
-
-
-def _extract_search_results(html_data: str) -> List:
+def _extract_pypi_search_results(html_data: str) -> List[Dict[str, str]]:
     from html.parser import HTMLParser
 
     def get_class(attrs):
@@ -1502,6 +1633,19 @@ def get_not_supported_translation():
     return tr("Package manager is not available for this interpreter")
 
 
+def download_dist_data_from_pypi(name: str, version: Optional[str]) -> Dict:
+    if version is None:
+        url = "https://pypi.org/pypi/{}/json".format(urllib.parse.quote(name))
+    else:
+        url = "https://pypi.org/pypi/{}/{}/json".format(
+            urllib.parse.quote(name), urllib.parse.quote(version)
+        )
+
+    logger.info("Downloading package info (%r, %r) from %s", name, version, url)
+
+    return download_and_parse_json(url)
+
+
 def load_plugin() -> None:
     def get_pip_gui_class():
         proxy = get_runner().get_backend_proxy()
