@@ -9,11 +9,24 @@ class RunDebugConfigurationPage(ConfigurationPage):
     def __init__(self, master):
         super().__init__(master)
 
+        self.add_checkbox(
+            "run.allow_running_unnamed_programs",
+            tr("Allow running unnamed programs"),
+            row=3,
+            columnspan=3,
+        )
+
         self.add_checkbox(
             "run.auto_cd",
             tr("Change working directory to script directory on Run / Debug"),
             row=5,
             columnspan=3,
+        )
+        self.add_checkbox(
+            "run.warn_module_shadowing",
+            tr("Warn if a user module shadows a library module"),
+            row=6,
+            columnspan=3,
             pady=(0, 20),
         )
 
