@@ -2,6 +2,7 @@
 """Adds commands for opening certain Thonny folders"""
 
 from thonny import THONNY_USER_DIR, get_workbench
+from thonny.languages import tr
 from thonny.ui_utils import open_path_in_system_file_manager
 
 
@@ -15,10 +16,10 @@ def load_plugin() -> None:
     get_workbench().add_command(
         "open_program_dir",
         "tools",
-        _("Open Thonny program folder..."),
+        tr("Open Thonny program folder..."),
         cmd_open_program_dir,
         group=110,
     )
     get_workbench().add_command(
-        "open_data_dir", "tools", _("Open Thonny data folder..."), cmd_open_data_dir, group=110
+        "open_data_dir", "tools", tr("Open Thonny data folder..."), cmd_open_data_dir, group=110
     )
